"use strict";

var websites;
var elements = {
	msgEl : document.querySelector(".message"),
	removeButton: document.getElementById("remove-button"),
}

function initRemove(){
	chrome.storage.local.get('websites', function(result){
		websites = result.websites;
		renderWebsites(result.websites);
	});
}

function renderWebsites(websites){
	var parentEl = document.querySelector(".websites-container");
	for(let i = 0; i < websites.length; i++){
		for(let x in websites[i]){
			parentEl.innerHTML += template(x, websites[i][x]);
		}
	}
	var websiteElements = document.querySelectorAll(".website");
	
	for(let i = 0; i < websiteElements.length; i++){
		websiteElements[i].addEventListener("click", selectWebsite);
	}
	
	elements.removeButton.addEventListener("click", function(){
		removeWebsites(websites);
	});
	
	function selectWebsite(){
		var className = this.className;
		if(className.indexOf("selected") > -1){
			this.classList.remove("selected");
		}
		else{
			this.classList.add("selected");
		}
	}
}

function template(title, url){
	var tmp = '<div class="website">' +
				'<div class="title">' + title.charAt(0).toUpperCase() + title.substr(1) + '</div>' +
				'<div class="url">' + url + '</div>' +
			  '</div>';
	
	return tmp;
}

function removeWebsites(websites){ // We are passing the existing websites and not the selected websites
	var selectedWebsiteTitleElements, selectedWebsiteTitles = [], selectedWebsiteElements; 
	
	showMsg("Please wait..do not refresh");
	disableButton();
	selectedWebsiteElements =  document.querySelectorAll(".selected");
	selectedWebsiteTitleElements = document.querySelectorAll(".selected > .title");
	for(let i = 0; i < selectedWebsiteTitleElements.length; i++){
		selectedWebsiteTitles.push(selectedWebsiteTitleElements[i].innerHTML.toLowerCase());
	}
	
	for(let x of selectedWebsiteTitles){
		deleteFromArray(websites, x);
	}
	chrome.storage.local.set({websites: websites}, function removeWebsites(){
		showMsg("Removed successfully.");
		setTimeout(function(){
			hideMsg();
		}, 3000);
		enableButton();
		for(let element of selectedWebsiteElements){
			element.remove();
		};
	});
	
	for(let key of selectedWebsiteTitles){
		chrome.storage.local.remove(key);
	}
	
}

function showMsg(msg){
	elements.msgEl.innerHTML = msg;
	elements.msgEl.style.display = "block";
}

function hideMsg(){
	elements.msgEl.classList.add("hide-msg");
	setTimeout(function(){
		elements.msgEl.innerHTML = "";
		elements.msgEl.style.display = "none";
		elements.msgEl.classList.remove("hide-msg");
	}, 300);
}

function disableButton(){
	elements.removeButton.classList.add("disabled");
}

function enableButton(){
	elements.removeButton.classList.remove("disabled");
}

function deleteFromArray(arr, key){
	for(let i = 0; i < arr.length; i++){
		for(let x in arr[i]){
			if(x == key){
				arr.splice(i, 1);
				return;
			}
		}
	}
}

initRemove();