"use strict";

var elements = {
	greetingElement: document.querySelector(".greeting"),
	headingElement: document.getElementById("website-name"),
	linkContainer: document.querySelector("#nav_websites"),
	filterDropDown: document.getElementById("filter-stats"),
	filterButton: document.getElementById("filter-button"),
    chartCanvasContainer: document.querySelector(".chart"),
	currentElement: document.querySelector(".stats-today > .stat-count"),
	currentElementLabel: document.querySelector(".stats-today > .stat-label"),
	previousElement: document.querySelector(".stats-yesterday > .stat-count"),
	previousElementLabel: document.querySelector(".stats-yesterday > .stat-label"),
	percentElement: document.querySelector(".stats-total > .stat-count"),
	percentElementLabel: document.querySelector(".stats-total > .stat-label"),
	chartLabelOne: document.getElementById("label-one"),
	chartLabelTwo: document.getElementById("label-two")
}


var dayLabels = ["00:00", "", "02:00", "", "04:00", "", "06:00", "", "08:00", "", "10:00", "", "12:00", "", "14:00", "", "16:00", "", "18:00", "", "20:00", "", "22:00", ""];
var monthLabels = ["1st", "", "", "4th", "", "", "7th", "", "", "10th", "", "", "13th", "", "", "16th", "", "", "19th", "", "", "22nd", "", "", "25th", "", "", "28th", "", "", "31st"];
var yearLabels = ["Jan", "Feb", "Mar", "Apr", "May", "June", "July", "Aug", "Sep", "Oct", "Nov", "Dec"];

function main(parentObj, website, type){
	chrome.storage.local.get(null, function(result){
		var parentObj = result;
		initUI(parentObj);
	});
}

function initUI(parentObj){
	var name = localStorage.getItem("name");
	if(!name){
		name = prompt("Please enter your first name : ");
		if(name){
			localStorage.setItem("name", name);
		}
		name = name || "User";
	}
	elements.greetingElement.innerHTML = "Hello, " + name;
	var isFirst = true, count = 0;
	if(parentObj.websites){
		for(let el of parentObj.websites){
			for(let key in el){
				elements.linkContainer.innerHTML += template(key.charAt(0).toUpperCase() + key.substr(1));
				count++;
			}
		}
	}
	
	//no website to watch
	if(!count){
		document.querySelector(".main-container").style.display = "none";
		document.querySelector(".error-msg").style.display = "block";
		return;
	}
	var firstEl = document.querySelector(".nav-link"), firstTitle;
	firstTitle = firstEl.getAttribute("data-title");
	firstEl.classList.add("nav-link-active");
	initStats(parentObj, firstTitle.toLowerCase(), "day");
	addListeners();
}

function template(title){
	var tmp = '<div class="nav-link" data-title="' + title + '">' + title + '</div>';
	return tmp;
}

function addListeners(){
	var linkElements = document.querySelectorAll("#nav_websites .nav-link");
	for(let el of linkElements){
		el.addEventListener("click", function(){
			var type = elements.filterDropDown.value;
			var title = this.getAttribute("data-title");
			document.querySelector(".nav-link-active").classList.remove("nav-link-active");
			this.classList.add("nav-link-active");
			chrome.storage.local.get(null, function(parentObj){
				document.getElementById("chart").remove();
				createNewCanvas();
				initStats(parentObj, title.toLowerCase(), type);
			});
		});
	}
	
	elements.filterButton.addEventListener("click", function(){
		var type = elements.filterDropDown.value;
		var title = elements.headingElement.innerHTML;
		chrome.storage.local.get(null, function(parentObj){
			document.getElementById("chart").remove();
			createNewCanvas();
			initStats(parentObj, title.toLowerCase(), type);
		});
	});
}

function createNewCanvas(){
	elements.chartCanvasContainer.innerHTML = '<canvas id="chart" height="400" width="850"></canvas>';
}

function initStats(parentObj, website, type){
	var data = getData(parentObj, website, type);
	generateChart(data.label, data.dataSetOne, data.dataSetTwo, type);
	displayStats(data.dataSetOne.total, data.dataSetTwo.total, website, type);	
}

function getData(parentObj, website, type){
    var date = new Date(), month, dateObj, resultObj = {};
    month = getMonthInWords(date.getMonth());
    dateObj = {
        date: date.getDate(),
        month: month,
        year: date.getFullYear()
    }
    var total = 0, tmp = {};
    switch(type){
        case "day":
			tmp = dayDataProcessor(parentObj, website);
            break;
		case "week":
			tmp = weekDataProcessor(parentObj, website);
			break;
		case "month":
			tmp = monthDataProcessor(parentObj, website);
			break;
		case "year":
			tmp = yearDataProcessor(parentObj, website);
			break;
    }
	return tmp;
}

function dayDataProcessor(parentObj, website){
	var dayData, processedDayData, tmp = {}, dateObj = getDateObjFromMs(Date.now());
	tmp.label = dayLabels;
	dayData = getDayData(dateObj);
	processedDayData = processDayData(dayData);
	tmp.dataSetOne = processedDayData;
	tmp.dataSetOne.label = "Today's usage";
	dayData = getDayData(getPreviousDay(Date.now()));
	processedDayData = processDayData(dayData);
	tmp.dataSetTwo = processedDayData;
	tmp.dataSetTwo.label = "Previous day's usage";
	function getPreviousDay(ms){
		var dayInMs = 24 * 60 * 60 * 1000;
		return getDateObjFromMs(ms - dayInMs);
	}
	
	function getDayData(dateObj){
		var dayArr = parentObj[website] && parentObj[website][dateObj.year] && parentObj[website][dateObj.year][dateObj.month] && parentObj[website][dateObj.year][dateObj.month][dateObj.date];
		if(!dayArr){
			return {
				total: 0,
				data: new Array(24).fill(0, 0, 24)
			};
		}
		var total = 0;
		for(let i = 0; i < 24; i++){
			total += dayArr[i];
		}
		return {
			total: total,
			data: dayArr
		}
	}

	function processDayData(dayData){
		var tmp, total = {}, mins;
		for(let i = 0; i < dayData.data.length; i++){
			dayData.data[i] = Math.round(dayData.data[i] / 60);
		}
		mins = Math.round(dayData.total / 60);
		total.hrs = Math.floor(mins / 60);
		total.mins = mins % 60;
		tmp = {
			total: total,
			dataArr: dayData.data
		}
		return tmp;
	}
	return tmp;
}

function weekDataProcessor(parentObj, website){
	var tmp = {};
	var dayInMs =  24 * 60 * 60 * 1000, currentWeekData, previousWeekData;
	tmp.label = labelGenerator();
	currentWeekData = getWeekData(Date.now());
	previousWeekData = getWeekData(Date.now() - 7 * dayInMs);
	tmp.dataSetOne = {
		label: "This week's usage",
		dataArr: currentWeekData.dataArr,
		total: currentWeekData.total
	};
	tmp.dataSetTwo = {
		label: "Previous week's usage",
		dataArr: previousWeekData.dataArr,
		total: previousWeekData.total
	};
	
	function getWeekData(ms){
		var dateObj, hrs, mins, dataArr = [], totalHrs = 0, totalMins = 0;
		for(let i = 1; i <= 7; i++){
			dateObj = getDateObjFromMs(ms);
			mins = getDayUsageMinutes(parentObj, dateObj, website);
			totalMins += mins;
			hrs = (mins / 60).toFixed(1);
			dataArr.push(hrs);
			ms = ms - dayInMs;
		}
		totalHrs = Math.floor(totalMins / 60);
		totalMins = totalMins % 60;
		return {
			dataArr: dataArr.reverse(),
			total:{
				hrs: totalHrs,
				mins: totalMins
			}
		};
	}
	
	function labelGenerator(){
		var dayNumber = new Date().getDay();
		var label = [];
		for(let i = 1; i <= 7; i++){
			label.push( getDayInWords(dayNumber + i) );
		}
		function getDayInWords(a){
		  switch(a)
		  {
			case 0:
			case 7:
			  return "Sun";
			case 1:
			case 8:
			  return "Mon";
			case 2:
		  	case 9:
			  return "Tue";
			case 3:
		  	case 10:
			  return "Wed";
			case 4:
		  	case 11:
			  return "Thu";
			case 5:
		  	case 12:
			  return "Fri";
			case 6:
		  	case 13:
			  return "Sat";
		  }
		}
		return label;
	}
	return tmp;
}

function monthDataProcessor(parentObj, website, type){
	var tmp = {};
	var currentDate, currentMonth, currentYear, previousMonth, previousYear, currentMonthData, previousMonthData;
	initVariables();
	currentMonthData = getMonthData({date: 1, month: currentMonth, year: currentYear});
	previousMonthData = getMonthData({date: 1, month: previousMonth, year: previousYear});
	tmp.label = monthLabels;
	tmp.dataSetOne = {
		label: "This month's usage",
		dataArr: currentMonthData.dataArr,
		total: currentMonthData.total
	};
	tmp.dataSetTwo = {
		label: "Previous month's usage",
		dataArr: previousMonthData.dataArr,
		total: previousMonthData.total
	};
	
	function initVariables(){
		currentDate = new Date();
		currentMonth = currentDate.getMonth();
		currentYear = currentDate.getFullYear();
		if(currentMonth == 0){
			previousMonth = 11;
			previousYear = currentYear - 1;
		}
		else{
			previousMonth = currentMonth - 1;
			previousYear = currentYear;
		}
		currentMonth = getMonthInWords(currentMonth);
		previousMonth = getMonthInWords(previousMonth);
	}
	
	function getMonthData(dateObj){
		var dailyUsageMinutes, dailyUsageHrs, dataArr = [], totalMins = 0, totalHrs;
		for(let i = 1; i <= 31; i++){
			dateObj.date = i;
			dailyUsageMinutes = getDayUsageMinutes(parentObj, dateObj, website);
			dailyUsageHrs = (dailyUsageMinutes / 60).toFixed(1);
			dataArr.push(dailyUsageHrs);
			totalMins += dailyUsageMinutes;
		}
		totalHrs = Math.floor(totalMins / 60);
		totalMins = totalMins % 60;
		return {
			dataArr: dataArr,
			total:{
				hrs: totalHrs,
				mins: totalMins
			}
		};	
	}
	return tmp;
}

function yearDataProcessor(parentObj, website, type){
	var tmp = {};
	var currentYear, previousYear, currentYearData, previousYearData;
	initVariables();
	var date = new Date();
	currentYearData = getYearData({date: 1, month: "JAN", year: currentYear});
	previousYearData = getYearData({date: 1, month: "JAN", year: previousYear});
	tmp.label = yearLabels;
	tmp.dataSetOne = {
		label: "This year's usage",
		dataArr: currentYearData.dataArr,
		total: currentYearData.total
	};
	tmp.dataSetTwo = {
		label: "Previous month's usage",
		dataArr: previousYearData.dataArr,
		total: previousYearData.total
	};
	
	function initVariables(){
		var date = new Date();
		currentYear = date.getFullYear();
		previousYear = currentYear - 1;
	}
	
	function getYearData(dateObj){
		var year = dateObj.year, month, monthlyUsageMins, monthlyUsageHrs, totalMins = 0, totalHrs, dataArr = [];
		for(let i = 0; i < 12; i++){
			month = getMonthInWords(i);
			monthlyUsageMins = getMonthUsageMins({date: 1, month: month, year: year});
			monthlyUsageHrs = (monthlyUsageMins / 60).toFixed(1);
			dataArr.push(monthlyUsageHrs);
			totalMins += monthlyUsageMins;
		}
		totalHrs = Math.floor(totalMins / 60);
		totalMins = totalMins % 60;
		return {
			dataArr: dataArr,
			total:{
				hrs: totalHrs,
				mins: totalMins
			}
		};
	}
	
	function getMonthUsageMins(dateObj){
		var totalMins = 0;
		for(let i = 1; i <= 31; i++){
			dateObj.date = i;
			totalMins += getDayUsageMinutes(parentObj, dateObj, website);
		}
		return totalMins;
	}
	return tmp;
}

function displayStats(totalObjOne, totalObjTwo, website, type){
	if(type == "day"){
		elements.currentElementLabel.innerHTML = "Spent today";
		elements.chartLabelOne.innerHTML = "Used today";
	}
	else{
		elements.currentElementLabel.innerHTML = "Spent this " + type;
		elements.chartLabelOne.innerHTML = "Used this " + type;
	}
	elements.headingElement.innerHTML = website.charAt(0).toUpperCase() + website.substr(1);
	elements.currentElement.innerHTML = "<span>" + totalObjOne.hrs + " hrs</span> <span>" + totalObjOne.mins + " mins</span>";
	elements.previousElement.innerHTML = "<span>" + totalObjTwo.hrs + " hrs</span> <span>" + totalObjTwo.mins + " mins</span>";
	elements.previousElementLabel.innerHTML = "Spent previous " + type;
	elements.chartLabelTwo.innerHTML = "Used previous " + type;
	var difference = (totalObjOne.hrs * 60 + totalObjOne.mins) - (totalObjTwo.hrs * 60 + totalObjTwo.mins), IncDec, percent;
	if((totalObjTwo.hrs * 60 + totalObjTwo.mins) !== 0){
		percent = Math.abs(difference * 100 / (totalObjTwo.hrs * 60 + totalObjTwo.mins));
	}
	else{
		percent = 100;
	}
	if(difference >= 0){
		IncDec = "Increase";
	}
	else{
		IncDec = "Decrease";
	}
	elements.percentElement.innerHTML = "<span>" + percent.toFixed(2) + "</span> <span>percent</span>";
	elements.percentElementLabel.innerHTML = IncDec + " in usage";
}

function generateChart(label, datasetOne, datasetTwo, type){
    var ctx = document.getElementById("chart").getContext("2d");
    var chartOptions = chartTemplate(label, datasetOne, datasetTwo, type); //pass parameters accordingly
    var myChart = new Chart(ctx, chartOptions);
}

function getDayUsageMinutes(parentObj, dateObj, website){
	var dayArr = parentObj[website] && parentObj[website][dateObj.year] && parentObj[website][dateObj.year][dateObj.month] && parentObj[website][dateObj.year][dateObj.month][dateObj.date];
	if(!dayArr){
		return 0;
	}
	var totalSeconds = 0;
	for(let i = 0; i < 24; i++){
		totalSeconds += dayArr[i]; //total in seconds
	}
	var totalMins = Math.round(totalSeconds / 60);
	return totalMins;
}

function getDateObjFromMs(ms){
	var date = new Date(ms);
	return {
		date: date.getDate(),
		month: getMonthInWords(date.getMonth()),
		year: date.getFullYear()
	}
}

function getMonthInWords(monthNumber){
    switch(monthNumber){
        case 0:
            return "JAN";
        case 1:
            return "FEB";
        case 2:
            return "MAR";
        case 3:
            return "APR";
        case 4:
            return "MAY";
        case 5:
            return "JUN";
        case 6:
            return "JUL";
        case 7:
            return "AUG";
        case 8:
            return "SEP";
        case 9:
            return "OCT";
        case 10:
            return "NOV";
        case 11:
            return "DEC";
    }
}
	
function chartTemplate(label, datasetOne, datasetTwo, type){
    var ctx = document.getElementById("chart").getContext("2d");
    var gradientOne = ctx.createLinearGradient(0, 0, 0, 400);
    gradientOne.addColorStop(0, "rgba(126, 184, 250, 0.1)");
    gradientOne.addColorStop(0.85, "rgba(126, 184, 250, 0.01)");
	var gradientTwo = ctx.createLinearGradient(0, 0, 0, 400);
    gradientTwo.addColorStop(0, "rgba(234, 76, 137, 0.1)");
    gradientTwo.addColorStop(0.85, "rgba(234, 76, 137, 0.01)");
	var xLabel, yLabel = "Time spent in hours";
	switch(type){
		case "day":
			xLabel = "Time in hh:mm format";
			yLabel = "Time spent in minutes";
			break;
		case "week":
			xLabel = "Days of the week";
			break;
		case "month":
			xLabel = "Date of the month";
			break;
		case "year":
			xLabel = "Months of the year";
   }
   var template = {
        type: 'line',
        data: {
            labels: label,
            datasets: [{
                label: datasetOne.label,
                data: datasetOne.dataArr,
                backgroundColor: gradientOne,
                borderColor: "#80b5e8",
                borderWidth: 2
            }, {
                label: datasetTwo.label,
                data: datasetTwo.dataArr,
                backgroundColor: gradientTwo,
                borderColor: "#fd38a5",
                borderWidth: 2
            }]
        },
        options: {
            legend: {
              display: false,
              position: "bottom",
              labels: {
                fontSize: 14,
                fontColor: "#333",
                fontFamily: "roboto",
                padding: 20
              }
            },
            layout: {
              padding: {
                right: 10,
                left: 10,
                bottom: 10,
                top: 10
              }
            },
            title: {
              display: false,
              text: 'Demo Title'
            },
            tooltips:{
              mode: "nearest",
              intersect: false,
              bodyFontSize: 14,
              titleSpacing: 2,
              xPadding: 10,
              yPadding: 10
            },
            scales: {
                xAxes: [{
                    gridLines: {
                      display: false,
                      color: "#eee"
                    },
					scaleLabel: {
						display: true,
						labelString: xLabel,
						fontSize: 14,
						fontColor: "#888"
					},
                    ticks: {
                        beginAtZero:true,
                        fontColor: "#888",
						padding: 10,
                    }
                }],
                yAxes: [{
                    gridLines: {
                      display: true,
                      color: "#eee",
                      borderDash: [],
                      drawTicks: false
                    },
					scaleLabel: {
						display: true,
						labelString: yLabel,
						fontSize: 14,
						fontColor: "#888"
					},
                    ticks: {
                        display: true,
                        beginAtZero:true,
                        maxTicksLimit: 5,
                        //max: 25,
                        fontColor: "#888",
                        padding: 15
                    }
                }]
            }
        }
    }
   
   return template;
}


main();